import commands
import os
import sys
import string

###############################################################################
def check_func_flag( context, flag ):
        context.Message( 'Checking for function flag "' + flag + '" ... ' )
        ret = context.TryCompile( """
""" + flag + """ int test (int a, int b)
{
        return a + b;
}
""", ".c" )
        context.Result( ret );
        return ret;


###############################################################################
def check_lflag( context, lflag, add = 1 ):
        context.Message( 'Checking to see if linker flag ' + lflag + ' works ... ' )
        old_flags = context.env[ 'LINKFLAGS' ]
        context.env[ 'LINKFLAGS' ] = lflag
        ret = context.TryCompile ("""
        int main (int argc, char *argv[]) {
                return 0;
        }
""", ".c")

        context.env[ 'LINKFLAGS' ] = old_flags

        if( ret and add ):
                context.env.Append( LINKFLAGS = [lflag] )

        context.Result( ret )
        return ret



###############################################################################
def check_lib( context, lib, add = 1 ):
        context.Message( 'Checking to see if library ' + repr( lib ) + ' is present ... ' )
        old_flags = context.env[ 'LIBS' ]
        context.env[ 'LIBS' ] = lib
        ret = context.TryLink ("""
        int main (int argc, char *argv[]) {
                return 0;
        }
""", ".c")

        context.env[ 'LIBS' ] = old_flags

        if( ret and add ):
                context.env.Append( LIBS = lib )

        context.Result( ret )
        return ret



###############################################################################
def check_cflag( context, cflag, add = 1 ):
        context.Message( 'Checking to see if compiler flag ' + cflag + ' works ... ' )
        old_flags = context.env[ 'CCFLAGS' ]
        context.env[ 'CCFLAGS' ] = cflag
        ret = context.TryCompile( """
        int main (int argc, char *argv[]) {
                return 0;
        }
""", ".c")

        context.env[ 'CCFLAGS' ] = old_flags

        if( ret and add ):
                context.env.Append( CCFLAGS = [cflag] )

        context.Result( ret )
        return ret



###############################################################################
def check_funcs( conf, defs, funcs ):
        for func in funcs:
                if conf.CheckFunc( func ):
                        defs.set('HAVE_' + string.upper(func), 1)



###############################################################################
def check_cheaders (conf, defs, headers):
        for header in headers:
                if conf.CheckCHeader( header ):
                        str = string.upper(header)
                        str = string.replace(str, '.', '_')
                        str = string.replace(str, '/', '_')
                        defs.set('HAVE_' + str, 1)



###############################################################################
def write_c_defines( env, filename, defs ):
        env.Append( CPPDEFINES = 'HAVE_CONFIG_H' )
        fh = open( str( filename ), "w" )
        for key in defs.keys():
                try:
                        fh.write( "#define " + key + "\t\t" + defs[key] + "\n" )
                except TypeError:
                        fh.write( "#define " + key + "\t\t" + str( defs[key] ) + "\n" )
        fh.close( );



###############################################################################
def check_config( context, command ):
        # the default parse function
        def parse_libs( env, output ):
                dict = {
                        'LIBPATH'   : [],
                        'LIBS'      : [],
                        'LINKFLAGS' : [],
                }
                static_libs = []

                params = string.split( output )
                for arg in params:
                        switch = arg[0:1]
                        opt = arg[1:2]
                        if switch == '-':
                                if opt == 'L':
                                        dict['LIBPATH'].append( arg[2:] )
                                elif opt == 'l':
                                        dict['LIBS'].append( arg[2:] )
                                else:
                                        dict['LINKFLAGS'].append( arg )
                        else:
                                static_libs.append( arg )
                apply( env.Append, (), dict )
                return static_libs

        # --- back to the top level function ---
        arguments = string.split( command )
        program = arguments.pop( 0 )
        args = arguments

        # if i am using pkg-config check for existance
        if program == 'pkg-config':
                package = args.pop()
                context.Message ('Checking for ' + package + ' ... ')
                file = os.popen( 'pkg-config --exists ' + package )
                if file.close() != None:
                        context.Result( 0 )
                        return (0, [])
                try:
                        pkg_ver_str = string.strip( os.popen( 'pkg-config --modversion ' + package ).read( ) )
                        pkg_ver = map( int, string.split( pkg_ver_str, "." ))
                        context.Result( repr( pkg_ver ) )
                except:
                        context.Result( 0 )
                        return (0, [])
                if "--libs" in args:
                        parse_libs( context.env, os.popen( "pkg-config --libs " + package ).read() )
                        args.remove( "--libs" )
                if len(args) > 0:
                        context.env.ParseConfig( "pkg-config " + string.join(args) + " " + package )
                return (1, pkg_ver)
        else:
                context.Message ('Checking for ' + program + ' ... ')
                if context.env.WhereIs( program ) == None:
                        context.Result( 0 )
                        return (0, [])
                try:
                        pkg_ver_str = string.strip( os.popen( program + ' --version' ).read( ) )
                        pkg_ver = map( int, string.split( pkg_ver_str, "." ) )
                        context.Result( repr( pkg_ver ) )
                except:
                        context.Result( 0 )
                        return (0, [])
                if "--libs" in args:
                        parse_libs( context.env, os.popen( program + " --libs" ).read() )
                        args.remove( "--libs" )
                if len(args) > 0:
                        context.env.ParseConfig( program + " " + string.join(args) )
                return (1, pkg_ver)
