import commands
import os
import sys
import string

#import BuildOptions class
from BuildOptions import *
from BuildCheck import *
from DevIL import *
from glut import *

build_opts = BuildOptions( )
build_defs = BuildOptions( )



###############################################################################
def conf_base( ):
        global build_opts
        build_opts.create( 'optimize', 0, 'Enable optimizations' )
        build_opts.create( 'CC', env[ 'CC' ], 'C compiler command' )
        build_opts.create( 'CFLAGS', '', 'Base C compiler flags' )

        return build_opts



###############################################################################
def handle_options( conf, build_opts, build_defs ):
        conf.env.Replace( CC = build_opts[ 'CC' ] )
        conf.env.Replace( CCFLAGS = Split( build_opts[ 'CFLAGS' ] ) )
        if( 'gcc' in env[ 'TOOLS' ] ):
                conf.cflag( '-g' )
                conf.cflag( '-Wall' )
                if int( build_opts[ 'optimize' ] ):
                        conf.cflag( '-O2' )
        if( 'msvc' in env[ 'TOOLS' ] ):
                if int( build_opts[ 'optimize' ] ):
                        conf.cflag( '/O2', 1 )
                        conf.cflag( '/Og', 1 )
                        conf.cflag( '/Oi', 1 )
                        conf.cflag( '/G5', 1 )




###############################################################################
def do_configure( env ):
        global build_opts, build_defs

        #set version in config.h
        build_defs.set( 'VERSION', '"0.0.1cvs"' )

        #read in options file
        if( not( ARGUMENTS.has_key( 'reset' ) and ARGUMENTS[ 'reset' ] ) ):
                build_opts.load( '../options.py' )
        build_opts.args( ARGUMENTS )
        build_opts.save( '../options.py' )

        #create configure context
        tests = { 'check_glut' : check_glut, 'check_devil' : check_devil, 'check_config' : check_config, 'cflag' : check_cflag, 'lflag' : check_lflag, 'func_flag' : check_func_flag, "lib" : check_lib }
        conf = Configure( env, custom_tests = tests )

        #pass options to configure
        handle_options( conf, build_opts, build_defs )

        #check for sdl
        if conf.check_config( 'sdl-config --libs --cflags' )[ 0 ]:
                build_opts.set( 'build_sdl', 1 )
                #build_defs.set( 'HAVE_SDL', 1 )
        else:
                build_opts.set( 'build_sdl', 0 )
                print "Simple Direct Media Layer (http://libsdl.org) is required (SDL)."
                Exit( 1 )

        #check for xine
        if conf.check_config( 'xine-config --libs --cflags' )[ 0 ]:
                build_opts.set( 'build_xine', 1 )
                build_defs.set( 'HAVE_XINE', 1 )
                env.Append( CPPPATH = ['/usr/include/xine'] ) #XXX
        else:
                build_opts.set( 'build_xine', 0 )

        #check for gtk
#        have_gtk = conf.check_config( 'pkg-config --libs --cflags gtk+-2.0 ' )[ 0 ]
#        have_gdkglext = conf.check_config( 'pkg-config --libs --cflags gdkglext-1.0 ' )[ 0 ]
#        have_gtkglext = conf.check_config( 'pkg-config --libs --cflags gtkglext-1.0 ' )[ 0 ]
#        if have_gtk and have_gdkglext and have_gtkglext:
#                build_opts.set( 'build_gtk', 1 )
#                build_defs.set( 'HAVE_GTK', 1 )
#        else:
#                build_opts.set( 'build_gtk', 0 )

        build_opts.set( 'build_gtk', 0 )

        #check for devil
        if conf.check_devil( )[ 0 ]:
                build_opts.set( 'build_devil', 1 )
                build_defs.set( 'HAVE_DEVIL', 1 )
                env.Append( LIBS = [ 'IL', 'ILU' ] )
        else:
                build_opts.set( 'build_devil', 0 )

        #check for gl
        env.Append( LIBPATH = ['/usr/X11R6/lib'] ) #XXX
        if not conf.lib( ['GL', 'GLU'] ):
           print "OpenGL is required (GL)."
           Exit( 1 )

        #check for glut
        if conf.check_glut( )[ 0 ]:
                build_opts.set( 'build_glut', 1 )
                #build_defs.set( 'HAVE_GLUT', 1 )
                env.Append( LIBS = [ 'Xi', 'Xmu' ] ) #XXX
                env.Append( LIBS = [ 'glut' ] )
        else:
                build_opts.set( 'build_glut', 0 )

        #figure out what makes the compiler to always honor "inline"
        if conf.func_flag('inline __attribute__((always_inline))'):
                build_defs.create('inline', 'inline __attribute__((always_inline))')
        elif conf.func_flag('inline'):
                build_defs.create('inline', 'inline')
        elif conf.func_flag('__inline'):
                build_defs.create('inline', '__inline')

        if env['PLATFORM'] == 'win32':
                env.Append (LIBS = ['user32', 'gdi32', 'shell32', 'wsock32', 'msvcrt', 'kernel32'])
                env.Append (LINKFLAGS = ['/subsystem:windows'])

        #finish with configure context
        conf.Finish( )

        #write out some useful info
        print "\nWriting src/config.h"
        write_c_defines( env, File( "#/src/config.h" ), build_defs )

        print """
  gcd v""" + build_defs.get('VERSION')[1:-1] + """ configuration:
    Build platform              : """ + env['PLATFORM'] + """
    Compiler                    : """ + env['CC'] + """
    Compiler flags              : """ + string.join(env['CCFLAGS'], " ")
        if (env.has_key ('LINKFLAGS') and env['LINKFLAGS']):
                print """\
    Link flags                  : """ + string.join(env['LINKFLAGS'], " ")
        print """\
    Libraries                   : """ + string.join(env['LIBS'], " ")
        if (env.has_key ('LIBPATH') and env['LIBPATH']):
                print """\
    Lib path                    : """ + string.join(env['LIBPATH'], " ")
        print """\
  """



###############################################################################
#main method
env = Environment( ENV = {'PATH' : os.environ['PATH']}, LIBS = 'm' )
conf_base( )
do_configure( env )
Export( "env", "build_opts" )
