//FILE:         args.h
//AUTHOR:       Nathan Cournia <nathan@cournia.com>

#ifndef COURNIA_ARGS_H
#define COURNIA_ARGS_H 1

#include <iostream>
#include <string>
#include "commandoptions.hpp"
#include "scale.h"
#include "gaze_backend.h"
#include "gl_mipmap.h"

#define ARGS_DEBUG 1
//#define ARGS_DEBUG 0

struct cmd_args
{
   unsigned int w, h;
   scale_t scale;
   bool fullscreen;
   std::string img_map;
   std::string lod_map;
   std::string fragment;
   std::string vertex;
   std::string device;
   gaze_backend gaze;
   gl_mipmap mipmap;
   std::string red_map, green_map, blue_map;

   cmd_args( void ): w( 640 ), h( 480 ),
      scale( SCALE_ASPECT ),
      fullscreen( false ),
      fragment( "data/degrade.fp" ),
      vertex( "data/pass.vp" ),
      device( "/dev/ttyS1" ),
      gaze( GAZE_MOUSE ),
      mipmap( GL_LINEAR_MIPMAP_LINEAR )
   { }

   bool parse_args( int argc, const char *argv[ ] ) {
      //command line parser object
      commandoptions c;

      //register flags/options
      c.register_option( w, "width", 'w', "Width of the window", "SIZE" );
      c.register_option( h, "height", 'h', "Height of the window", "SIZE" );
      c.register_option( scale, "scale", 's', "Scale image", "ASPECT,STRETCH,NONE" );
      c.register_flag( fullscreen, "fs", 'f', "Fullscreen" );
      c.register_option( fragment, "fragment", 'p', "Fragment program", "FILE" );
      c.register_option( vertex, "vertex", 'v', "Vertex program", "FILE" );
      c.register_option( gaze, "gaze", 'z', "Gaze backend", "MOUSE,ISCAN" );
      c.register_option( mipmap, "mipmap", 'm', "Mipmapping Bias", "GL_*_MIPMAP_*" );
      c.register_option( red_map, "red", 'r', "Red Channel", "IMAGE" );
      c.register_option( green_map, "green", 'g', "Green Channel", "IMAGE" );
      c.register_option( blue_map, "blue", 'b', "Blue Channel", "IMAGE" );
      c.register_option( device, "device", 'd', "Eye tracker device", "FILE" );
      c.register_argument( img_map, "image", "Input image" );
      c.register_argument( lod_map, "lod", "Input LOD image" );

      try {
         c.process_command_line( argc, argv );
      } catch( commandoptions_error& ex ) {
         std::cerr << "error: " << ex.what( ) << std::endl;
         return false;
      }

      //success
      return true;
   }
};

#endif
