//FILE:         devil.cpp
//AUTHOR:       Nathan Cournia <nathan@cournia.com>

#include <IL/il.h>
#include <IL/ilu.h>

#include "devil.h"
#include "gl_utils.h"

//#define DEBUG(x) (std::cerr << __FILE__ << ":" << __LINE__ << " " << __PRETTY_FUNCTION__ << ": " << x << std::endl)
#define DEBUG(x)

////////////////////////////////////////////////////////////////////////////////
rgb_image*
devil_load_rgb( const std::string& filename )
{
   //init devil
   static bool initd = false;
   if( !initd ) {
      ilInit( );
      iluInit( );
      initd = true;
   }

   //generate and bind an image name
   ILuint img;
   ilGenImages( 1, &img );
   ilBindImage( img );

   //load the input image then check for error
   //FIXME il.h says that ilLoadImage takes a const char*
   //      but the compiler thinks that it takes a char*
   if( ilLoadImage( const_cast<char*>( filename.c_str( ) ) ) == IL_FALSE ) {
      //report error
      std::cerr << "error: " << iluErrorString( ilGetError( ) ) << std::endl;
      return 0;
   }

   //alloc memory for image
   rgb_image *dst = new rgb_image;
   dst->resize( 
      ilGetInteger( IL_IMAGE_HEIGHT ),
      ilGetInteger( IL_IMAGE_WIDTH )
   );

   //copy image from devil
   ilCopyPixels( 0, 0, 0, dst->w, dst->h, 1, IL_RGB, IL_UNSIGNED_BYTE, dst->pixels );

   //remove image from devil
   ilDeleteImages( 1, &img );

   //return new image
   return dst;
}



////////////////////////////////////////////////////////////////////////////////
grey_image*
devil_load_grey( const std::string& filename )
{
   //init devil
   static bool initd = false;
   if( !initd ) {
      ilInit( );
      iluInit( );
      initd = true;
   }

   //generate and bind an image name
   ILuint img;
   ilGenImages( 1, &img );
   ilBindImage( img );

   //load the input image then check for error
   //FIXME il.h says that ilLoadImage takes a const char*
   //      but the compiler thinks that it takes a char*
   if( ilLoadImage( const_cast<char*>( filename.c_str( ) ) ) == IL_FALSE ) {
      //report error
      std::cerr << "error: " << iluErrorString( ilGetError( ) ) << std::endl;
      return 0;
   }

   //alloc memory for image
   grey_image *dst = new grey_image;
   dst->resize( 
      ilGetInteger( IL_IMAGE_HEIGHT ),
      ilGetInteger( IL_IMAGE_WIDTH )
   );

   //copy image from devil
   ilCopyPixels( 0, 0, 0, dst->w, dst->h, 1, IL_LUMINANCE, IL_UNSIGNED_BYTE, dst->pixels );

   //remove image from devil
   ilDeleteImages( 1, &img );

   //return new image
   return dst;
}



////////////////////////////////////////////////////////////////////////////////
gl_texture
devil_load_rgb_gl( const std::string& filename )
{
   rgb_image *img = devil_load_rgb( filename );
   if( !img ) return gl_texture( );

   img->flip_horizontal( );

   gl_texture handle = gl_load( img );
   delete img;;
   return handle;
}



////////////////////////////////////////////////////////////////////////////////
gl_texture
devil_load_grey_gl( const std::string& filename )
{
   grey_image *img = devil_load_grey( filename );
   if( !img ) return gl_texture( );

   img->flip_horizontal( );

   gl_texture handle = gl_load( img );
   delete img;;
   return handle;
}
