//FILE:         error_base.h
//AUTHOR:       Nathan Cournia <nathan@cournia.com>

#ifndef CLEMSON_ERROR_BASE_H
#define CLEMSON_ERROR_BASE_H 1

#include <string>
#include <cstdio>
#include <cstdarg>

//! Error reporting base class.
class error_base
{
 protected:
	std::string m_error_msg; //!< Error message.
	bool m_error; //!< \c true if object is in an error state.

	//! Clears the internal error state.
	inline void clear_error( void ) {
		m_error = false;
		m_error_msg = "none";
	}

	//! Sets the internal error state.
	inline void set_error( const std::string& msg ) {
		m_error = true;
		m_error_msg = msg;
	}

	//! Sets internal error state (printf style).
	inline void set_errorf( const char *fmt, ... ) {
		char buffer[ 1024 ];
		va_list ap;
		va_start( ap, fmt );
		vsnprintf( buffer, 1024, fmt, ap );
		va_end( ap );
		m_error = true;
		m_error_msg = buffer;
	}

 public:
	//! Default constructor.
	error_base( void ): m_error_msg( "none" ), m_error( false ) { }

	//! Default destructor.
	virtual ~error_base( void ) { }

	//! Returns the error message.
	inline std::string get_error( void ) {
		//copy error message (clear_error erases it)
		std::string msg( m_error_msg );
		//clear_error( );
		return msg;
	}

	//! Overload \c bool operator.  Returns \c false if the object is in an error state.
	virtual operator bool ( void ) const {
		//std::cerr << "bool: error: " << m_error << " returning " << !m_error << std::endl;
		return !m_error;
	}

	//! Returns \c true if in error state.
	virtual bool error( void ) const {
		return m_error;
	}
};

#endif
