//FILE:         gaze_backend.cpp
//AUTHOR:       Nathan Cournia <nathan@cournia.com>

#include "config.h"
#include "gaze_backend.h"
#include "args.h"
#include "gaze_driver.h"
#include "gaze_iscan.h"
#include "gaze_sdl.h"
#include "gaze_glut.h"

///////////////////////////////////////////////////////////////////////////////
std::istream& 
operator>>( std::istream& in, gaze_backend& backend )
{
   std::string upper;
   in >> upper;
   std::transform( upper.begin( ), upper.end( ), upper.begin( ),
      static_cast<int(*)(int)>(toupper) );
   if( "MOUSE" == upper ) backend = GAZE_MOUSE;
   else if( "ISCAN" == upper ) backend = GAZE_ISCAN;
   else backend = GAZE_MOUSE;
   return in;
}



///////////////////////////////////////////////////////////////////////////////
gaze_driver*
init_gaze( const cmd_args& args )
{
   //locals
   gaze_iscan *piscan;
   switch( args.gaze ) {
      case GAZE_ISCAN:
         piscan = new gaze_iscan( args.device, args.w, args.h );
         if( piscan ) piscan->start( );
         return piscan;
      break;

      default:
#if HAVE_SDL
         return new gaze_sdl( args.w, args.h );
#elif HAVE_GLUT
         return new gaze_glut( args.w, args.h );
#else
         return 0;
#endif
   }
}
