//FILE:         gaze_iscan.h
//AUTHOR:       Nathan Cournia <nathan@cournia.com>

#ifndef GAZE_ISCAN_H
#define GAZE_ISCAN_H 1

#include "iscan.h"

class gaze_mapper: public iscan::mapper
{
 public:
   gaze_mapper( real_t xmin, real_t xmax, 
      real_t ymin, real_t ymax ): 
      m_xmin( xmin ), m_xmax( xmax ), 
      m_ymin( ymin ), m_ymax( ymax ) 
   { }

   ~gaze_mapper() { }

   real_t map_x( real_t before ) const { 
      //std::cerr << "x: " << before << std::endl;
      return map_linear( m_xmin, m_xmax, before );
   }

   real_t map_y( real_t before ) const { 
      //std::cerr << "y: " << before << std::endl;
      return 1.0 - map_linear( m_ymin, m_ymax, before ); 
   }

 private:
   inline real_t map_linear( real_t min, real_t max, 
      real_t before ) const { 
      return (before - min) / (max - min); 
   }

   real_t m_xmin;
   real_t m_xmax;
   real_t m_ymin;
   real_t m_ymax;
};



class gaze_iscan: public gaze_driver
{
 public:
   class iscan_callback: public iscan::update_callback
   {
    public:
      iscan_callback( gaze_iscan *gaze ): m_gaze( gaze ) { }

      virtual void operator()( iscan* ) {
         m_gaze->set_update( true );
      }

    private:
      gaze_iscan *m_gaze;
   };

   gaze_iscan( const std::string& port, int w, int h ):
      m_callback( this ),
      m_width( w ), m_height( h ),
      m_update( false )
   {
      m_iscan.open( port, iscan::MONOCULAR );
      assert( m_iscan );

      m_iscan.get_left_eye( ).set_mapper( 
         new gaze_mapper( 0.0, 512.0, 0.0, 512.0 ) );

      m_iscan.set_update_callback( &m_callback );
   }

   void start( void ) {
      m_iscan.start( );
   }

   void set_update( bool up ) {
      m_update = up;
   }

   virtual bool need_update( void ) {
      if( m_update ) {
         m_update = false;
         return true;
      }
      return false;
   }

   virtual void get_gaze( int& x, int &y ) {
      real_t ex, ey;
      m_iscan.get_left_eye( ).get_x( ex );
      m_iscan.get_left_eye( ).get_y( ey );
      x = static_cast<int>( ex * m_width );
      y = static_cast<int>( ey * m_height );
      //std::cerr << "eye: " << ex << " " << ey << std::endl;
   }

 private:
   iscan_callback m_callback;
   int m_width, m_height;
   bool m_update;
   iscan m_iscan;
};

#endif
