//FILE:         gaze_view.cpp
//AUTHOR:       Nathan Cournia <nathan@cournia.com>

#include "gaze_view.h"

///////////////////////////////////////////////////////////////////////////////
void
gcd_view::scale_none( void )
{
   int cx = window_width / 2;
   int cy = window_height / 2;
   int tcx = texture.get_width( ) / 2;
   int tcy = texture.get_height( ) / 2;
   x = cx - tcx;
   y = cy - tcy;
   w = texture.get_width( );
   h = texture.get_height( );
}



///////////////////////////////////////////////////////////////////////////////
void
gcd_view::scale_stretch( void )
{
   x = 0;
   y = 0;
   w = window_width;
   h = window_height;
}



///////////////////////////////////////////////////////////////////////////////
void
gcd_view::scale_aspect( void )
{
   int cx = window_width / 2;
   int cy = window_height / 2;
   float ta = static_cast<float>( texture.get_width( ) ) / texture.get_height( );
   float wa = static_cast<float>( window_width ) / window_height;

   if( ta > wa ) {
      x = 0; w = window_width;
      ta = static_cast<float>( window_width ) / texture.get_width( );
      h = static_cast<int>( ta * texture.get_height( ) );
      y = cy - h / 2;
   } else {
      y = 0; h = window_height;
      ta = static_cast<float>( window_height ) / texture.get_height( );
      w = static_cast<int>( ta * texture.get_width( ) );
      x = cx - w / 2;
   }
}



///////////////////////////////////////////////////////////////////////////////
//assumes we are in an orthgraphic projection
void
gcd_view::render( void )
{
   //bind the texture
   glColor4f( 1.0, 1.0, 1.0, 1.0 );
   texture.enable_bind( );
   glTexParameteri( texture.get_target( ), GL_TEXTURE_MIN_FILTER, mipmap_mode );

   lod_texture.enable_bind( 1 );

   //bind the vertex program
   vp.enable_bind( );

   //bind the fragment program
   fp.enable_bind( );

   //set the eye location
   //fp.set_local_parameter( 0, eye_x, eye_y, lods, 0.0 );
   fp.set_local_parameter( 0, lods, 0.0, 0.0, 0.0 );

   float lx = eye_x - x - static_cast<float>( lod_texture.get_width( ) ) / 2.0;
   float ly = eye_y - y - static_cast<float>( lod_texture.get_height( ) ) / 2.0;
   lx /= -static_cast<float>( lod_texture.get_width( ) );
   ly /= -static_cast<float>( lod_texture.get_height( ) );
   //std::cerr << lx << " " << ly << std::endl;

   float lw = static_cast<float>( w ) / lod_texture.get_width( );
   float lh = static_cast<float>( h ) / lod_texture.get_height( );

   //hack for video images (which are loaded into opengl upside down)
   float flip = 0.0;
   if( should_flip ) {
      flip = 1.0;
   }

   glBegin( GL_QUADS );
      //lower left
      glMultiTexCoord2fARB( GL_TEXTURE0_ARB, 0.0, 0.0 + flip );
      glMultiTexCoord2fARB( GL_TEXTURE1_ARB, lx, ly );
      glVertex2f( x, y );

      //lower right
      glMultiTexCoord2fARB( GL_TEXTURE0_ARB, 1.0, 0.0 + flip );
      glMultiTexCoord2fARB( GL_TEXTURE1_ARB, lx + lw, ly );
      glVertex2f( x + w, y );

      //upper right
      glMultiTexCoord2fARB( GL_TEXTURE0_ARB, 1.0, 1.0 - flip );
      glMultiTexCoord2fARB( GL_TEXTURE1_ARB, lx + lw, ly + lh );
      glVertex2f( x + w, y + h );

      //upper left
      glMultiTexCoord2fARB( GL_TEXTURE0_ARB, 0.0, 1.0 - flip );
      glMultiTexCoord2fARB( GL_TEXTURE1_ARB, lx, ly + lh );
      glVertex2f( x, y + h );
   glEnd( );

   //unbind
   fp.disable( );
   vp.disable( );
   lod_texture.disable( 1 );
   texture.disable( );
}
