//FILE:         gaze_view.h
//AUTHOR:       Nathan Cournia <nathan@cournia.com>

#ifndef GAZE_VIEW_H
#define GAZE_VIEW_H 1

#include "extgl.h"
#include "args.h"
#include "gl_utils.h"
#include "gl_fragment_program.h"
#include "gl_vertex_program.h"
#include "mathematics.h"

class gcd_view
{
 public:
   gcd_view( gl_texture tex, gl_texture lod, gl_fragment_program& frag, 
      gl_vertex_program& vert, const cmd_args& args ):
      texture( tex ),
      lod_texture( lod ),
      fp( frag ),
      vp( vert ),
      eye_x( 0 ), eye_y( 0 ),
      should_flip( false ),
      scale_mode( args.scale ),
      window_width( args.w ),
      window_height( args.h ),
      mipmap_mode( GL_LINEAR_MIPMAP_LINEAR )
   {
      scale( );
   }

   virtual ~gcd_view( void ) { }

   void scale( void ) {
      switch( scale_mode ) {
         case SCALE_STRETCH:
            scale_stretch( );
         break;

         case SCALE_ASPECT:
            scale_aspect( );
         break;

         case SCALE_NONE:
            scale_none( );
         break;

         default:
            scale_none( );
         break;
      }
      //std::cerr << "w: " << texture.get_width( ) << " h: " << texture.get_height( ) << std::endl;
      //std::cerr << "x: " << x << " y: " << y << " w: " << w << " h: " << h << std::endl;

      //compute number of level of details
      lods = std::max( texture.get_width( ), texture.get_height( ) );
      lods = power_of_two( lods );
      lods = static_cast<int>( log( lods ) / log( 2 ) );
      //std::cerr << "lods: [0-" << lods << "]" << std::endl;
   }

   void set_texture( gl_texture new_tex ) {
      if( (texture.get_width( ) != new_tex.get_width( )) ||
          (texture.get_height( ) != new_tex.get_height( )) ) {
         //dimension mismatch, rescale gcd
         texture = new_tex;
         scale( );
      } else {
         texture = new_tex;
      }
   }

   void set_eye( int x, int y ) {
      eye_x = x;
      eye_y = y;
   }

   void render( void );

   void set_mipmapping( GLenum mipmap ) {
      mipmap_mode = mipmap;
   }

   void set_flip( bool x ) {
      should_flip = x;
   }

 protected:
   void scale_none( void );

   void scale_stretch( void );

   void scale_aspect( void );

   gl_texture texture;
   gl_texture lod_texture;
   gl_fragment_program& fp;
   gl_vertex_program& vp;
   float eye_x, eye_y;
   int x, y, w, h;
   int lods;
   bool should_flip;
   scale_t scale_mode;
   int window_width, window_height;
   GLenum mipmap_mode;
};



#endif
