//FILE:         gl_program.h
//AUTHOR:       Nathan Cournia <nathan@cournia.com>

#ifndef UBER_GL_PROGRAM_H
#define UBER_GL_PROGRAM_H 1

#include "extgl.h"
#include "vector.h"

#include <map>
#include <string>

class gl_program
{
 public:
   virtual ~gl_program( void );

   virtual bool enable_bind( void );

   virtual bool disable( void );

   virtual bool compile( void );

   virtual void set_local_parameter( unsigned int index, real_t s );
   
   virtual void set_local_parameter( unsigned int index, 
      real_t x, real_t y, real_t z, real_t w );

   virtual void set_local_parameter( unsigned int index, 
      const vector& v, real_t w );

   virtual void erase( void );

   static std::string load( const std::string& filename );

   static std::string load( std::istream& input );

   bool error( void ) {
      return m_error;
   }

 protected:
   gl_program( GLenum gl_program_type, const std::string& code, 
      bool should_compile );

   void upload_local_param( unsigned int param );

   struct vec4 {
      real_t data[ 4 ];
      void set( real_t x_, real_t y_, real_t z_, real_t w_ ) {
         data[ 0 ] = x_; data[ 1 ] = y_; data[ 2 ] = z_; data[ 3 ] = w_;
      }
      operator const real_t* ( void ) const {
         return data;
      }
   };
   typedef std::map<unsigned int, vec4> param_list;

   GLenum m_program_type;
   unsigned int m_handle;
   std::string m_code;
   param_list m_locals;
   bool m_is_bound;
   bool m_error;
   
   //do not define
   gl_program( const gl_program& cpy );
   gl_program& operator=( const gl_program& cpy );
};

#endif
