//FILE:         gl_texture.h
//AUTHOR:       Nathan Cournia <nathan@cournia.com>

#ifndef COURNIA_TEXTURE_H
#define COURNIA_TEXTURE_H 1

#include "extgl.h"

class gl_texture
{
 public:
   gl_texture( void ): m_handle( 0 ),
       m_target( GL_TEXTURE_RECTANGLE_EXT ),
       m_width( 0 ), m_height( 0 )
   { }

   virtual ~gl_texture( void ) { }

   inline void set_handle( GLuint handle ) {
      m_handle = handle;
   }

   inline void set_target( GLenum target ) {
      m_target = target;
   }

   inline void set_width( unsigned int w ) {
      m_width = w;
   }

   inline void set_height( unsigned int h ) {
      m_height = h;
   }

   inline int get_width( void ) const {
      return m_width;
   }

   inline int get_height( void ) const {
      return m_height;
   }

   inline GLenum get_target( void ) const {
      return m_target;
   }

   inline void erase( void ) {
      if( m_handle != 0 ) {
         glDeleteTextures( 1, &m_handle );
      }
   }

   inline void bind( void ) const {
      glBindTexture( m_target, m_handle );
   }

   inline void enable_bind( unsigned int unit = 0 ) const {
      glActiveTextureARB( GL_TEXTURE0_ARB + unit );
      glEnable( m_target );
      glBindTexture( m_target, m_handle );
   }

   inline void disable( unsigned int unit = 0 ) const {
      glActiveTextureARB( GL_TEXTURE0_ARB + unit );
      glDisable( m_target );
   }

   bool error( void ) const {
      return (0 == m_handle);
   }

 protected:
   GLuint m_handle;
   GLenum m_target;
   unsigned int m_width;
   unsigned int m_height;
};

#endif
