//FILE:         mathematics.h
//AUTHOR:       Nathan Cournia <nathan@cournia.com>

#ifndef COURNIA_MATHEMATICS_H
#define COURNIA_MATHEMATICS_H 1 

#include <cstdlib>
#include <cmath>

typedef float real_t;

//#define MATH_EPSILON 1.19209290e-07f
#define MATH_EPSILON 0.00001

//////////////////////////////////////////////////////////////////////////
//return random number between [0-1]
inline double
rand01( void ) 
{
   return (rand( ) / (double)RAND_MAX);
}

//////////////////////////////////////////////////////////////////////////
//convert radians to degrees
inline double
to_degrees( double x )
{
   return (x * 180.0 / M_PI);
}

//////////////////////////////////////////////////////////////////////////
//convert degrees to radians
inline double
to_radians( double x )
{
   return (x * M_PI / 180.0);
}

//////////////////////////////////////////////////////////////////////////
inline bool
equals( double x, double y ) 
{
   return (fabs( x - y ) < MATH_EPSILON);
}

///////////////////////////////////////////////////////////////////////////////
inline double 
min( double x, double y )
{
   if( x < y ) {
      return x;
   } else {
      return y;
   }
}

///////////////////////////////////////////////////////////////////////////////
inline double 
max( double x, double y )
{
   if( x > y ) {
      return x;
   } else {
      return y;
   }   
}

///////////////////////////////////////////////////////////////////////////////
inline void
swap( double& x, double& y )
{
   double z = x;
   x = y;
   y = z;
}

///////////////////////////////////////////////////////////////////////////////
inline double
sign( double x )
{
   if( x < 0.0 ) {
      return -1.0;
   } else {
      return 1.0;
   }
}

///////////////////////////////////////////////////////////////////////////////
inline bool
same_sign( double x, double y )
{
   if( x * y >= 0.0 ) {
      return true;
   } else {
      return false;
   }
}

///////////////////////////////////////////////////////////////////////////////
inline bool
negative( double x )
{
   if( x < 0.0 ) {
      return true;
   } else {
      return false;
   }
}

///////////////////////////////////////////////////////////////////////////////
inline bool
positive( double x )
{
   if( x < 0.0 ) {
      return false;
   } else {
      return true;
   }
}

///////////////////////////////////////////////////////////////////////////////
template <class T>
T 
clamp( T value, T min, T max )
{
   if( value < min ) {
      //value less, clamp
      return min;
   }

   if( value > max ) {
      //value greater, clamp
      return max;
   }

   //value ok
   return value;
}


///////////////////////////////////////////////////////////////////////////////
inline int
power_of_two( int x )
{
   int value = 1;
   while( value < x ) {
      value <<= 1;
   }
   return value;
}

///////////////////////////////////////////////////////////////////////////////
inline bool
is_power_of_two( unsigned long n )
{
   return (n != 0) && ((n & (n - 1)) == 0);
}

#endif
