//FILE:         vector.cpp
//AUTHOR:       Nathan Cournia <nathan@cournia.com>

#include <cassert>
#include "vector.h"

//////////////////////////////////////////////////////////////////////////
vector 
operator+ ( const vector& lhs, const vector& rhs )
{
   return vector( (lhs.m_data[ 0 ] + rhs.m_data[ 0 ]), 
                 (lhs.m_data[ 1 ] + rhs.m_data[ 1 ]),
                  (lhs.m_data[ 2 ] + rhs.m_data[ 2 ])
                );
}

//////////////////////////////////////////////////////////////////////////
vector 
operator- ( const vector& lhs, const vector& rhs )
{
   return vector( (lhs.m_data[ 0 ] - rhs.m_data[ 0 ]), 
                 (lhs.m_data[ 1 ] - rhs.m_data[ 1 ]),
                  (lhs.m_data[ 2 ] - rhs.m_data[ 2 ])
                );
}

//////////////////////////////////////////////////////////////////////////
vector 
operator- ( const vector& lhs )
{
   return vector( (lhs.m_data[ 0 ] * -1.0), 
                 (lhs.m_data[ 1 ] * -1.0),
                  (lhs.m_data[ 2 ] * -1.0)
                );
}

//////////////////////////////////////////////////////////////////////////
vector 
operator* ( const vector& lhs, real_t rhs )
{
   return vector( (lhs.m_data[ 0 ] * rhs), 
                 (lhs.m_data[ 1 ] * rhs),
                  (lhs.m_data[ 2 ] * rhs)
                );
}

//////////////////////////////////////////////////////////////////////////
vector 
operator* ( real_t rhs, const vector& lhs )
{
   return vector( (lhs.m_data[ 0 ] * rhs), 
                 (lhs.m_data[ 1 ] * rhs),
                  (lhs.m_data[ 2 ] * rhs)
                );
}

//////////////////////////////////////////////////////////////////////////
vector
operator/ ( const vector& lhs, real_t rhs )
{
   return vector( (lhs.m_data[ 0 ] / rhs), 
                 (lhs.m_data[ 1 ] / rhs),
                  (lhs.m_data[ 2 ] / rhs)
                );
}

//////////////////////////////////////////////////////////////////////////
vector 
cross_new( const vector& u, const vector& v )
{
   return vector( u.m_data[ 1 ] * v.m_data[ 2 ] - u.m_data[ 2 ] * v.m_data[ 1 ],
                  u.m_data[ 2 ] * v.m_data[ 0 ] - u.m_data[ 0 ] * v.m_data[ 2 ],
                  u.m_data[ 0 ] * v.m_data[ 1 ] - u.m_data[ 1 ] * v.m_data[ 0 ] 
                );
}

//////////////////////////////////////////////////////////////////////////
std::ostream& 
operator<< ( std::ostream& o, const vector& vec )
{
   return o << vec.m_data[ 0 ] << " " << vec.m_data[ 1 ] 
      << " " << vec.m_data[ 2 ];
}

//////////////////////////////////////////////////////////////////////////
void 
add( vector& dest, const vector& lhs, const vector& rhs )
{
   dest.m_data[ 0 ] = lhs.m_data[ 0 ] + rhs.m_data[ 0 ];
   dest.m_data[ 1 ] = lhs.m_data[ 1 ] + rhs.m_data[ 1 ];
   dest.m_data[ 2 ] = lhs.m_data[ 2 ] + rhs.m_data[ 2 ];
}

//////////////////////////////////////////////////////////////////////////
void 
add( vector& dest, const vector& rhs )
{
   dest.m_data[ 0 ] += rhs.m_data[ 0 ];
   dest.m_data[ 1 ] += rhs.m_data[ 1 ];
   dest.m_data[ 2 ] += rhs.m_data[ 2 ];
}

//////////////////////////////////////////////////////////////////////////
void 
subtract( vector& dest, const vector& lhs, const vector& rhs )
{
   dest.m_data[ 0 ] = lhs.m_data[ 0 ] - rhs.m_data[ 0 ];
   dest.m_data[ 1 ] = lhs.m_data[ 1 ] - rhs.m_data[ 1 ];
   dest.m_data[ 2 ] = lhs.m_data[ 2 ] - rhs.m_data[ 2 ];
}

//////////////////////////////////////////////////////////////////////////
void 
subtract( vector& dest, const vector& rhs )
{
   dest.m_data[ 0 ] -= rhs.m_data[ 0 ];
   dest.m_data[ 1 ] -= rhs.m_data[ 1 ];
   dest.m_data[ 2 ] -= rhs.m_data[ 2 ];
}

//////////////////////////////////////////////////////////////////////////
void 
negate( vector& dest )
{
   dest.m_data[ 0 ] *= -1.0;
   dest.m_data[ 1 ] *= -1.0;
   dest.m_data[ 2 ] *= -1.0;
}

//////////////////////////////////////////////////////////////////////////
void 
multiply( vector& dest, const vector& lhs, real_t rhs )
{
   dest.m_data[ 0 ] = lhs.m_data[ 0 ] * rhs;
   dest.m_data[ 1 ] = lhs.m_data[ 1 ] * rhs;
   dest.m_data[ 2 ] = lhs.m_data[ 2 ] * rhs;
}

//////////////////////////////////////////////////////////////////////////
void 
multiply( vector& dest, real_t rhs, const vector& lhs )
{
   dest.m_data[ 0 ] = lhs.m_data[ 0 ] * rhs;
   dest.m_data[ 1 ] = lhs.m_data[ 1 ] * rhs;
   dest.m_data[ 2 ] = lhs.m_data[ 2 ] * rhs;
}

//////////////////////////////////////////////////////////////////////////
void 
multiply( vector& dest, real_t rhs )
{
   dest.m_data[ 0 ] *= rhs;
   dest.m_data[ 1 ] *= rhs;
   dest.m_data[ 2 ] *= rhs;
}

//////////////////////////////////////////////////////////////////////////
real_t 
dot( const vector& lhs, const vector& rhs )
{
   return ( 
            (lhs.m_data[ 0 ] * rhs.m_data[ 0 ]) +
           (lhs.m_data[ 1 ] * rhs.m_data[ 1 ]) +
            (lhs.m_data[ 2 ] * rhs.m_data[ 2 ])
          );
}
   
//////////////////////////////////////////////////////////////////////////
//FIXME needs to be optimized
void 
cross( vector& dest, const vector& rhs )
{
   dest = cross_new( dest, rhs );
}

//////////////////////////////////////////////////////////////////////////
vector::vector( void )
{
   m_data[ 0 ] = 0.0;
   m_data[ 1 ] = 0.0;
   m_data[ 2 ] = 0.0;
}

//////////////////////////////////////////////////////////////////////////
vector::vector( real_t x_, real_t y_, real_t z_ )
{
   m_data[ 0 ] = x_;
   m_data[ 1 ] = y_;
   m_data[ 2 ] = z_;
}

//////////////////////////////////////////////////////////////////////////
vector::vector( const vector& p )
{
   m_data[ 0 ] = p.m_data[ 0 ];
   m_data[ 1 ] = p.m_data[ 1 ];
   m_data[ 2 ] = p.m_data[ 2 ];
}

//////////////////////////////////////////////////////////////////////////
vector&
vector::operator= ( const vector& p )
{
   m_data[ 0 ] = p.m_data[ 0 ];
   m_data[ 1 ] = p.m_data[ 1 ];
   m_data[ 2 ] = p.m_data[ 2 ];
   return *this;
}

//////////////////////////////////////////////////////////////////////////
void
vector::set( real_t x_, real_t y_, real_t z_ )
{
   m_data[ 0 ] = x_;
   m_data[ 1 ] = y_;
   m_data[ 2 ] = z_;
}

//////////////////////////////////////////////////////////////////////////
real_t
vector::operator[ ] ( unsigned int index ) const
{
   assert( index < 3 );
   return m_data[ index ];
}

//////////////////////////////////////////////////////////////////////////
real_t&
vector::operator[ ] ( unsigned int index )
{
   assert( index < 3 );
   return m_data[ index ];
}

//////////////////////////////////////////////////////////////////////////
void
vector::operator+= ( const vector& v )
{
   m_data[ 0 ] += v.m_data[ 0 ];
   m_data[ 1 ] += v.m_data[ 1 ];
   m_data[ 2 ] += v.m_data[ 2 ];
}

//////////////////////////////////////////////////////////////////////////
void
vector::operator*= ( real_t rhs )
{
   m_data[ 0 ] *= rhs;
   m_data[ 1 ] *= rhs;
   m_data[ 2 ] *= rhs;
}
