//FILE:         vector.h
//AUTHOR:       Nathan Cournia <nathan@cournia.com>

#ifndef COURNIA_VECTOR_H
#define COURNIA_VECTOR_H 1

#include <iostream>
#include <cmath>
#include "mathematics.h"

//////////////////////////////////////////////////////////////////////////
class vector;

//////////////////////////////////////////////////////////////////////////
//slow friends
vector operator+ ( const vector& lhs, const vector& rhs );
vector operator- ( const vector& lhs, const vector& rhs );
vector operator- ( const vector& v );
vector operator* ( const vector& lhs, real_t rhs );
vector operator* ( real_t lhs, const vector& rhs );
vector operator/ ( const vector& lhs, real_t rhs );
vector cross_new( const vector& lhs, const vector& rhs );
std::ostream& operator<< ( std::ostream& o, const vector& vec );

//////////////////////////////////////////////////////////////////////////
//fast friends
void add( vector& dest, const vector& lhs, const vector& rhs );
void add( vector& dest, const vector& rhs );
void multiply( vector& dest, const vector& lhs, real_t rhs );
void multiply( vector& dest, real_t rhs, const vector& lhs );
void multiply( vector& dest, real_t rhs );
void subtract( vector& dest, const vector& lhs, const vector& rhs );
void subtract( vector& dest, const vector& rhs );
void negate( vector& dest );
real_t dot( const vector& lhs, const vector& rhs ); 
void cross( vector& dest, const vector& rhs );

//////////////////////////////////////////////////////////////////////////
class vector
{
protected:
   real_t m_data[ 3 ];

   //////////////////////////////////////////////////////////////////////////
   //slow friends
   friend vector operator+ ( const vector& lhs, const vector& rhs );
   friend vector operator- ( const vector& lhs, const vector& rhs );
   friend vector operator- ( const vector& v );
   friend vector operator* ( const vector& lhs, real_t rhs );
   friend vector operator* ( real_t lhs, const vector& rhs );
   friend vector operator/ ( const vector& lhs, real_t rhs );
   friend vector cross_new( const vector& lhs, const vector& rhs );
   friend std::ostream& operator<< ( std::ostream& o, const vector& vec );

   //////////////////////////////////////////////////////////////////////////
   //fast friends
   friend void add( vector& dest, const vector& lhs, 
      const vector& rhs );
   friend void add( vector& dest, const vector& rhs );
   friend void multiply( vector& dest, const vector& lhs, real_t rhs );
   friend void multiply( vector& dest, real_t rhs, const vector& lhs );
   friend void multiply( vector& dest, real_t rhs );
   friend void subtract( vector& dest, const vector& lhs, 
      const vector& rhs );
   friend void subtract( vector& dest, const vector& rhs );
   friend void negate( vector& dest );
   friend real_t dot( const vector& lhs, const vector& rhs ); 
   friend void cross( vector& dest, const vector& rhs );

public:
   //////////////////////////////////////////////////////////////////////
   vector( void );
   
   //////////////////////////////////////////////////////////////////////
   vector( real_t x_, real_t y_, real_t z_ );

   //////////////////////////////////////////////////////////////////////
   vector( const vector& p );
   
   //////////////////////////////////////////////////////////////////////
   vector& operator= ( const vector& p );

   //////////////////////////////////////////////////////////////////////
   void set( real_t x_, real_t y_, real_t z_ );

   //////////////////////////////////////////////////////////////////////
   real_t operator[ ] ( unsigned int index ) const;

   //////////////////////////////////////////////////////////////////////
   real_t& operator[ ] ( unsigned int index );

   //////////////////////////////////////////////////////////////////////
   void operator+= ( const vector& v );

   //////////////////////////////////////////////////////////////////////
   void operator*= ( real_t rhs );
   
   //////////////////////////////////////////////////////////////////////
   inline real_t x( void ) const { 
      return m_data[ 0 ];
   }
   
   //////////////////////////////////////////////////////////////////////
   inline real_t y( void ) const { 
      return m_data[ 1 ];
   }

   //////////////////////////////////////////////////////////////////////
   inline real_t z( void ) const { 
      return m_data[ 2 ];
   }
   
   //////////////////////////////////////////////////////////////////////
   inline void x( real_t x_ ) {
      m_data[ 0 ] = x_;
   }
   //////////////////////////////////////////////////////////////////////
   inline void y( real_t y_ ) {
      m_data[ 1 ] = y_;
   }
   
   //////////////////////////////////////////////////////////////////////
   inline void z( real_t z_ ) {
      m_data[ 2 ] = z_;
   }

   //////////////////////////////////////////////////////////////////////
   inline double length( void ) const {
      return sqrt( (m_data[ 0 ] * m_data[ 0 ]) +
                  (m_data[ 1 ] * m_data[ 1 ]) +
                  (m_data[ 2 ] * m_data[ 2 ])
                  );
   }

   ///////////////////////////////////////////////////////////////////////
   inline void normalize( void ) {
      double len = vector::length( );
      m_data[ 0 ] /= len;
      m_data[ 1 ] /= len;
      m_data[ 2 ] /= len;
   }
};
   
#endif
