//FILE:         xine_dump.h
//AUTHOR:       Nathan Cournia <nathan@cournia.com>

#ifndef COURNIA_XINE_DUMP_H
#define COURNIA_XINE_DUMP_H 1

#include <string>
#include <xine.h>
#include "error_base.h"

extern "C"
{
   void xine_rgbout_callback( unsigned int width, unsigned int height, 
                              void *frame, void *data );
}

class xine_dump_callback
{
 public:
   virtual ~xine_dump_callback( void ) { }

   virtual void operator()( unsigned int width, unsigned int height,
                            unsigned char *pixels ) = 0;
};

class xine_dump: public error_base
{
 public:
   xine_dump( void );

   ~xine_dump( void );

   bool init( void );

   bool open( const std::string& filename );

   bool play( void );

   void stop( void );

   bool is_playing( void );

   bool is_finished( void );

   void set_finished( bool tf );

   void set_callback( xine_dump_callback *cb );

   bool skip( int seconds );

 private:
   void config_lookup( const char *id, int val, xine_cfg_entry_t *entry );

   void call_callback( unsigned int width, unsigned int height, void *pixels );

   xine_t *m_xine;
   xine_stream_t *m_stream;
   xine_vo_driver_t *m_vo_driver;
   xine_ao_driver_t *m_ao_driver;
   xine_event_queue_t *m_event_queue;

   xine_dump_callback *m_callback;

   bool m_finished;

   friend void xine_rgbout_callback( unsigned int width, unsigned int height, 
                                     void *frame, void *data );
};

#endif
